/* lsngp.c *************************************** UPDATED: 2000-04-30 01:17 TT
 *
 * Copyright   : (C) 2000 Tormod Tjaberg
 * Author      : Tormod Tjaberg (tjaberg@online.no)
 * Created     : 2000-04-29 
 * Description : LiSt contents og Neo Geo Pocket rom dump
 *
 * Note:
 * Information gleaned from various sources specially Anarko & Judge
 *
 * If you have any other info about the header please contact me.
 *
 * Changes:
 * 1.01 : 2000-04-30 : Added size of rom 
 * 1.00 : 2000-04-29 : Initial release
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <assert.h>

#define PROGRAM_NAME "lsngp"
#define PROGRAM_VER  "1.01"

/* Clever little option mimic wildcard expansion from
 * shell ONLY available with the Zortech compiler ver 3.0x and upwards
 */
#if defined(__ZTC__) 
extern int _cdecl __wildcard; int *__wild = &__wildcard;
#endif

char *Usage[] =
{
   PROGRAM_NAME ", ver " PROGRAM_VER " -- Copyright (C) 2000 Tormod Tjaberg",
   "",
   "Usage: " PROGRAM_NAME " <filename(s)>",
   "",
   "Displays header information from a NeoGeo Pocket rom dump",
#if defined(__ZTC__) || defined(__DJGPP__)
   "Wildcards in the filename are allowed.",
#endif
   NULL
};

/* Abort()
 *
 * Input  : pMessage - Message to be displayed with the format of standard printf!
 * Returns: Nothing, but instead it exits to operating system
 *
 * Terminate program, return to OS with a suitable return value.
 * exit() will close all open files.
 */
void Abort(char *pMessage, ...)
{
   va_list argp;

   fprintf(stderr, PROGRAM_NAME ": ");
   va_start(argp, pMessage);
   vfprintf(stderr ,pMessage, argp);
   va_end(argp);
   fprintf(stderr, "\n");
   exit(EXIT_FAILURE);  /* exit & tell operating system that we've failed */

}

/* Same as Abort we don't exit... 
 */
void Warning(char *pMessage, ...)
{
   va_list argp;

   fprintf(stderr, PROGRAM_NAME ": ");
   va_start(argp, pMessage);
   vfprintf(stderr ,pMessage, argp);
   va_end(argp);
   fprintf(stderr, "\n");
}

typedef struct
{
   char Copyright[28];  
   unsigned long ProgCount; /* Code starts at 0x200000 */
   unsigned short GameNum;
   unsigned char Unknown;
   unsigned char Type;
   char Title[12];
   char Filler[16];   /* should always be 0 */
} NGP_HDR;

NGP_HDR NGP_Hdr;

/* Sneaky little assert trick */
typedef int assert_dummy[sizeof(NGP_HDR) == 64];

int main(int argc, char *argv[])
{
   FILE *fp;
   char *pFileSpec;
   int i;
   int j;
   long l;
   int Mbit;

   if (argc < 2)
   {
      i = 0;
      while (Usage[i] != NULL)
         puts(Usage[i++]);
      return 0;
   }

   for (j = 1; j < argc; j++)
   {
      pFileSpec = argv[j];

      if ((fp = fopen(pFileSpec, "rb")) == NULL) 
      {
         Warning("unable to open input file: %s\n", pFileSpec);
         continue;
      }

      if (fread(&NGP_Hdr, sizeof(NGP_HDR), 1, fp) != 1)
      {
         fclose(fp);
         Warning("file too short to be an NGP rom: %s", pFileSpec);
         continue;
      }

      fseek(fp, 0, SEEK_END);
      l = ftell(fp);


      /* close file */
      fclose(fp);
      
      /* Sanity Check
       * . Title has to be printable
       * . Code has to start past 0x200000
       * . Type must be 0x10 (Colour) or 0x00 (Black/White)
       * . Filler bytes must be 0
       */
      if (!isprint(NGP_Hdr.Title[0]) || NGP_Hdr.ProgCount < 0x200000 ||
          NGP_Hdr.Filler[0] != 0 || NGP_Hdr.Filler[1] != 0)
      {
         Warning("%s: is NOT a valid NGP rom", pFileSpec);
         continue;
      }

      if (NGP_Hdr.Type != 0x10 && NGP_Hdr.Type != 0x0)
      {
         Warning("%s: header has invalid type: 0x%x", pFileSpec, NGP_Hdr.Type);
         continue;
      }

      Mbit = (l / 1024) / 128;

      printf("%s: %s: \"%s\" (#%d) %s %d Mbit\n", PROGRAM_NAME, pFileSpec, NGP_Hdr.Title, 
              NGP_Hdr.GameNum, NGP_Hdr.Type == 0x10 ? "Colour" : "Black/White", Mbit);
   }

   return 0;
}
